@@ -8,7 +8,6 @@ Author: Josef Perktold
 Created on Fri Jun  5 16:32:00 2015
 """
 
-from __future__ import division
 # import useful only for development
 from abc import ABCMeta, abstractmethod
 from statsmodels.compat.python import with_metaclass
@@ -45,7 +44,7 @@ def _R_compat_quantile(x, probs):
 def _eval_bspline_basis(x, knots, degree, deriv='all', include_intercept=True):
     try:
         from scipy.interpolate import splev
-    except ImportError:  # pragma: no cover
+    except ImportError:
         raise ImportError("spline functionality requires scipy")
     # 'knots' are assumed to be already pre-processed. E.g. usually you
     # want to include duplicate copies of boundary knots; you should do
@@ -59,7 +58,7 @@ def _eval_bspline_basis(x, knots, degree, deriv='all', include_intercept=True):
         x = x[:, 0]
     assert x.ndim == 1
     # XX FIXME: when points fall outside of the boundaries, splev and R seem
-    # to handle them differently. I don't know why yet. So until we understand
+    # to handle them differently. I do not know why yet. So until we understand
     # this and decide what to do with it, I'm going to play it safe and
     # disallow such points.
     if np.min(x) < np.min(knots) or np.max(x) > np.max(knots):
@@ -69,11 +68,11 @@ def _eval_bspline_basis(x, knots, degree, deriv='all', include_intercept=True):
     # Thanks to Charles Harris for explaining splev. It's not well
     # documented, but basically it computes an arbitrary b-spline basis
     # given knots and degree on some specificed points (or derivatives
-    # thereof, but we don't use that functionality), and then returns some
+    # thereof, but we do not use that functionality), and then returns some
     # linear combination of these basis functions. To get out the basis
     # functions themselves, we use linear combinations like [1, 0, 0], [0,
     # 1, 0], [0, 0, 1].
-    # NB: This probably makes it rather inefficient (though I haven't checked
+    # NB: This probably makes it rather inefficient (though I have not checked
     # to be sure -- maybe the fortran code actually skips computing the basis
     # function for coefficients that are zero).
     # Note: the order of a spline is the same as its degree + 1.
@@ -141,7 +140,6 @@ def get_knots_bsplines(x=None, df=None, knots=None, degree=3,
 
     The first corresponds to splines as used by patsy. the second is the
     knot spacing for P-Splines.
-
     """
     # based on patsy memorize_finish
     if all_knots is not None:
@@ -401,28 +399,28 @@ class UnivariateBSplines(UnivariateGamSmoother):
 
     Parameters
     ----------
-    x : array, 1-D
+    x : ndarray, 1-D
         underlying explanatory variable for smooth terms.
     df : int
-        numer of basis functions or degrees of freedom
+        number of basis functions or degrees of freedom
     degree : int
         degree of the spline
     include_intercept : bool
         If False, then the basis functions are transformed so that they
         do not include a constant. This avoids perfect collinearity if
         a constant or several components are included in the model.
-    constraints : None, string or array
+    constraints : {None, str, array}
         Constraints are used to transform the basis functions to satisfy
         those constraints.
         `constraints = 'center'` applies a linear transform to remove the
         constant and center the basis functions.
-    variable_name : None or str
+    variable_name : {None, str}
         The name for the underlying explanatory variable, x, used in for
         creating the column and parameter names for the basis functions.
-    covder2_kwds : None or dict
+    covder2_kwds : {None, dict}
         options for computing the penalty matrix from the second derivative
         of the spline.
-    knot_kwds : None or list of dict
+    knot_kwds : {None, list[dict]}
         option for the knot selection.
         By default knots are selected in the same way as in patsy, however the
         number of knots is independent of keeping or removing the constant.
@@ -443,7 +441,6 @@ class UnivariateBSplines(UnivariateGamSmoother):
         - all_knots : None or array
           If all knots are provided, then those will be taken as given and
           all other options will be ignored.
-
     """
     def __init__(self, x, df, degree=3, include_intercept=False,
                  constraints=None, variable_name='x',
@@ -476,7 +473,7 @@ class UnivariateBSplines(UnivariateGamSmoother):
 
         Parameters
         ----------
-        x_new : array
+        x_new : ndarray
             observations of the underlying explanatory variable
         deriv : int
             which derivative of the spline basis to compute
@@ -616,17 +613,17 @@ class UnivariateCubicCyclicSplines(UnivariateGamSmoother):
 
     Parameters
     ----------
-    x : array, 1-D
+    x : ndarray, 1-D
         underlying explanatory variable for smooth terms.
     df : int
-        numer of basis functions or degrees of freedom
+        number of basis functions or degrees of freedom
     degree : int
         degree of the spline
     include_intercept : bool
         If False, then the basis functions are transformed so that they
         do not include a constant. This avoids perfect collinearity if
         a constant or several components are included in the model.
-    constraints : None, string or array
+    constraints : {None, str, array}
         Constraints are used to transform the basis functions to satisfy
         those constraints.
         `constraints = 'center'` applies a linear transform to remove the
@@ -671,10 +668,16 @@ class UnivariateCubicCyclicSplines(UnivariateGamSmoother):
 
         Returns
         -------
-        b, d: ndarrays
-            arrays for mapping cyclic cubic spline values at knots to
+        b : ndarray
+            Array for mapping cyclic cubic spline values at knots to
             second derivatives.
-            penalty matrix is equal to ``s = d.T.dot(b^-1).dot(d)``
+        d : ndarray
+            Array for mapping cyclic cubic spline values at knots to
+            second derivatives.
+
+        Notes
+        -----
+        The penalty matrix is equal to ``s = d.T.dot(b^-1).dot(d)``
         """
         h = knots[1:] - knots[:-1]
         n = knots.size - 1
@@ -781,15 +784,16 @@ class AdditiveGamSmoother(with_metaclass(ABCMeta)):
 
         Parameters
         ----------
-        x_new: array
+        x_new: ndarray
             observations of the underlying explanatory variable
 
         Returns
         -------
         basis : ndarray
             design matrix for the spline basis for given ``x_new``.
-
         """
+        if x_new.ndim == 1 and self.k_variables == 1:
+            x_new = x_new.reshape(-1, 1)
         exog = np.hstack(list(self.smoothers[i].transform(x_new[:, i])
                          for i in range(self.k_variables)))
         return exog
@@ -837,20 +841,23 @@ class BSplines(AdditiveGamSmoother):
         underlying explanatory variable for smooth terms.
         If 2-dimensional, then observations should be in rows and
         explanatory variables in columns.
-    df :  int
-        numer of basis functions or degrees of freedom
-    degree : int
-        degree of the spline
+    df :  {int, array_like[int]}
+        number of basis functions or degrees of freedom; should be equal
+        in length to the number of columns of `x`; may be an integer if
+        `x` has one column or is 1-D.
+    degree : {int, array_like[int]}
+        degree(s) of the spline; the same length and type rules apply as
+        to `df`
     include_intercept : bool
         If False, then the basis functions are transformed so that they
         do not include a constant. This avoids perfect collinearity if
         a constant or several components are included in the model.
-    constraints : None, string or array
+    constraints : {None, str, array}
         Constraints are used to transform the basis functions to satisfy
         those constraints.
         `constraints = 'center'` applies a linear transform to remove the
         constant and center the basis functions.
-    variable_names : None or list of strings
+    variable_names : {list[str], None}
         The names for the underlying explanatory variables, x used in for
         creating the column and parameter names for the basis functions.
         If ``x`` is a pandas object, then the names will be taken from it.
@@ -902,13 +909,17 @@ class BSplines(AdditiveGamSmoother):
     support anymore. This is obtained ``constraints='center'``. In this case
     ``include_intercept`` will be automatically set to True to avoid
     dropping an additional column.
-
-
     """
     def __init__(self, x, df, degree, include_intercept=False,
                  constraints=None, variable_names=None, knot_kwds=None):
-        self.degrees = degree
-        self.dfs = df
+        if isinstance(degree, int):
+            self.degrees = np.array([degree], dtype=int)
+        else:
+            self.degrees = degree
+        if isinstance(df, int):
+            self.dfs = np.array([df], dtype=int)
+        else:
+            self.dfs = df
         self.knot_kwds = knot_kwds
         # TODO: move attaching constraints to super call
         self.constraints = constraints
@@ -938,7 +949,6 @@ class CubicSplines(AdditiveGamSmoother):
 
     Note, these splines do NOT use the same spline basis as
     ``Cubic Regression Splines``.
-
     """
     def __init__(self, x, df, constraints='center', transform='domain',
                  variable_names=None):
@@ -974,14 +984,13 @@ class CyclicCubicSplines(AdditiveGamSmoother):
         explanatory variables in columns.
     df :  int
         numer of basis functions or degrees of freedom
-    constraints : None, string or array
+    constraints : {None, str, array}
         Constraints are used to transform the basis functions to satisfy
         those constraints.
-    variable_names : None or list of strings
+    variable_names : {list[str], None}
         The names for the underlying explanatory variables, x used in for
         creating the column and parameter names for the basis functions.
         If ``x`` is a pandas object, then the names will be taken from it.
-
     """
     def __init__(self, x, df, constraints=None, variable_names=None):
         self.dfs = df
